open Mcic

let rec dump_expr_list el =
  match el with
      hd::tl -> dump_expr hd; dump_expr_list tl
    | [] -> ignore []
and dump_expr expr =
  match expr with
      Funcall( e1, e2 )-> print_string "func "; dump_expr e1; print_string "(";
	dump_expr_list e2; print_endline ")";
    | Int_Constant i -> print_int i; print_newline()
    | Str_Constant str -> print_endline ("\"" ^ str ^ "\"")
    | ID i -> print_endline i;  
    | Tuple el -> print_endline "tuple("; dump_expr_list el; print_endline")"
    | Stmt_Expr s -> print_endline "stmt_expr"; dump_stmt s
    | Asign_Expr( e1, e2 ) -> dump_expr e1; print_string " = ";dump_expr e2


and dump_stmt stmt =
  match stmt with
      A_Stmt e -> dump_expr e
    | IfStmt( e, s ) -> dump_expr e ; dump_stmt_list s
    | BlockStmt (s,symtab) -> print_endline "begin"; dump_stmt_list s; print_endline "end"
    | LoopStmt s -> dump_stmt_list s
    | DeclVarStmtInit( t, i, e ) -> 
	dump_expr t; print_string ("ident "^i); dump_expr e;
    | DeclVarStmt( t, i ) ->
	dump_expr t; print_string ("ident "^i)
    | DeclFunc( ret, id, args, stmts ) ->
	print_endline("defun");
	dump_expr ret;
	dump_stmt_list stmts;
	print_endline("end defun")

and dump_stmt_list sl =
  match sl with
      hd::tl -> dump_stmt hd; dump_stmt_list tl
    | [] -> ignore [] ;;

let dump_program prog =
  dump_stmt_list prog;;

