open Env;;

type label = string;;

type emit_opration = 
    EmitString of string
  | EmitMakeFrame;;

let emit_buffer = ref [];;

let push_back v = 
  emit_buffer.contents<-(List.append !emit_buffer (v::[]));;

let emit_make_frame () =
  push_back EmitMakeFrame;;

let emit_remove_frame () =
  push_back (EmitString("\tmovl\t%ebp, %esp\n\tpopl\t%ebp\n"));;

let emit_string str =
  push_back (EmitString(str^"\n"));;

let emit_sectname name =
  push_back (EmitString("\t.section "^name^"\n" ));;

let emit_label label = 
  push_back (EmitString(label^":\n"));;

let emit_globl_label label =
  push_back (EmitString("\t"^".globl "^label^"\n")); 
  emit_label label;;

let emit_func_end frame_size = 
  List.iter (fun op -> 
	       match op with
		   EmitString str -> output_string (output())str
		 | EmitMakeFrame -> 
		     output_string (output()) (
		     "\tpush\t%ebp\n\tmovl\t%esp, %ebp\n"^
		     "\tsubl\t$"^(string_of_int frame_size)^", %esp\n")
	    ) !emit_buffer;
  output_string (output()) "\tret\n";
  emit_buffer.contents <- [];;

let emit_inst s = ();;

exception InternalError of string;;

let emit_flush () =
  List.iter ( fun op ->
		match op with 
		    EmitString str -> output_string (output()) str
		  | _-> raise (InternalError("flush")) ) !emit_buffer;
  emit_buffer.contents <- [];;

let emit_comment s = emit_string ("#"^s);;

let emit_storage size =
  emit_string ("\t.zero\t"^(string_of_int size));;

let emit_long init =
  emit_string ("\t.long\t"^(string_of_int init));;

let emit_align num = 
  emit_string ("\t.align\t"^(string_of_int num));;

let emit_bss () =
  emit_string "\t.bss";;

let emit_data () =
  emit_string "\t.data";;

let emit_text () =
  emit_string "\t.text";;

let emit_rodata () = 
  emit_string "\t.section\t.rodata";;
