
type type_expr =
    IntType
  | CharType
  | FuncType of type_expr * type_expr list
  | VoidType
  | ErrorType
  | VarArgType
  | Pointer_type of type_expr 
  | Array_type of type_expr * int
  | TypeType of type_expr 

class variable (id_:string) (typ_:type_expr) (_global:bool)=
object (self)
  method id = id_
  method typ = typ_

  val mutable offset = 0;
  method get_offset () = offset;
  method set_offset off = offset <- off;

  method is_global = _global

end;;

exception Redefined of string;;
exception InternalError of string;;

type symtab_type = GlobalTable | FunctionTable | LocalTable | NamespaceTable;;

class symbol_table (table_type:symtab_type)=
object ( self )
  val table =  Hashtbl.create 133 
  method regist (var:variable) =
    let id = (var#id) in
      if self#mem id then
	raise (Redefined id)
      else
	Hashtbl.add table id var
  method get (id:string) =
    Hashtbl.find table id
  method mem (id:string) =
    Hashtbl.mem table id

  val mutable parents = [];
  method set_parents (p:symbol_table list) = parents <- p;
  method get_parents () = parents;

  val mutable size = 0;
  val mutable max_size = 0;

  method expand num =
    if self#get_type = LocalTable then
      (List.hd (self#get_parents()))#expand num
    else
      (size <- size+num;
       if size > max_size then
	 max_size <- size
       else
	 ())

  method contract num =
    size <- size - num;

  method inc_size num =
    if self#get_type = LocalTable then
      (size <- size + num;
       (List.hd (self#get_parents()))#expand num)
    else
      self#expand num

  method align num =
    let xor n = (~-n) in 
    let diff = (size+(num-1)) land (xor num)-size
    in
      self#inc_size diff;
      self#get_size();

  method get_size () = size;
  method get_max_size () = max_size;

  method get_type = table_type;

end

let env_stack = ref [];;
let get_env_stack () = !env_stack;;
type ret_type_holder = { mutable content: type_expr; };;
let ret_type = ref { content = IntType; };;

let find_id id = List.find ( fun tbl -> tbl#mem id ) (get_env_stack());;

let current_rettype() = !ret_type.content;;
let set_rettype typ = !ret_type.content <- typ;;

let current () = List.hd !env_stack;;

let push (tbl:symbol_table) = 
  env_stack.contents <- tbl::!env_stack;;

let pop () = 
  env_stack.contents <- List.tl !env_stack;;

let reg_num=6;;

let registers = Array.create reg_num true;;

let is_reg_usable num = registers.(num);;

let usable_reg () =			(* Ȥ쥸 *)
    let rec check num = 
      if num >= reg_num then
	-1
      else
	if registers.(num) then
	  num
	else
	  check (num+1)
    in
      check 0;;

let use_reg num = registers.(num) <- false;;
let unuse_reg num = registers.(num) <- true;;

type chan = { mutable content: out_channel; };;

let output_file = ref {content=stdout;};;
let output () = 
  !output_file.content;;

let set_output out =
  !output_file.content <- out;;

let exit_label = ref "";;

let set_exit_label l = exit_label.contents <- l;;
let get_exit_label () = !exit_label;;

let local_label_count = ref 0;;

let gen_local_label ()=
  local_label_count.contents <- (!local_label_count+1);
  ".__LC"^(string_of_int !local_label_count)

let nul_compile str = ();;

let compile_func_value = ref nul_compile;;

let set_compile_func func = compile_func_value.contents <- func;;
let compile_func () = !compile_func_value;;
