exception InternalError of string;;

type type_expr =
    IntType
  | CharType
  | FuncType of type_expr * type_expr list
  | VoidType
  | ErrorType 
  | VarArgType
  | Pointer_type of type_expr 
  | Array_type of type_expr * int
  | TypeType of type_expr

class variable :
  string -> type_expr -> bool ->
object
  method id: string
  method typ: type_expr

  method set_offset: int -> unit
  method get_offset: unit -> int

  method is_global: bool
end

type symtab_type = GlobalTable | FunctionTable | LocalTable | NamespaceTable;;

class symbol_table :
  symtab_type -> 
object
  method regist :
    variable -> unit
  method get :
    string -> variable
  method mem:
    string -> bool
  method inc_size: 
    int -> unit
  method align:
    int -> int
  method get_size:
    unit -> int
  method get_max_size:
    unit -> int
  method get_type : 
    symtab_type
  method get_parents:
    unit -> symbol_table list
  method set_parents :
    symbol_table list -> unit

  method expand : 
    int->unit
  method contract :
    int->unit
end;;

val get_env_stack: unit -> symbol_table list;;
val current: unit->symbol_table;;

(* IDơ֥֤ *)
val find_id: string -> symbol_table;;

val current_rettype: unit->type_expr;;
val set_rettype: type_expr->unit;;

val push: symbol_table -> unit;;
val pop: unit->unit;;

val set_output: out_channel -> unit;;

val use_reg: int -> unit;;
val unuse_reg: int -> unit;;

val usable_reg: unit->int;;
val is_reg_usable: int -> bool;;

val output: unit->out_channel;;

val set_exit_label: string->unit;;
val get_exit_label: unit->string;;

val gen_local_label: unit->string;;

val set_compile_func: (string->unit)->unit;;
val compile_func: unit->(string->unit);;
