{
open Parser;;
exception Error of string;;
exception InvalidChar of char;;

let keyword_table =
  Util.create_hashtbl 53
    ["begin",BEGIN;
     "end",END;
     "loop",LOOP;
     "if",IF;
     "else",ELSE;
     "then",THEN;
     "return",RETURN;
     "extern",EXTERN;
     "while",WHILE;
     "elsif",ELSIF;
     "do",DO;
     "done",DONE;
     "import",IMPORT;];;

let nowchar lexbuf =
  Lexing.lexeme_char lexbuf 0 
;;

}

let decimal_literal = ['0'-'9']['0'-'9']*
let blank = [' ' '\009']
let newline = ['\010' '\013']
let id_word = ['a'-'z' 'A'-'Z' '_' '0'-'9']+
let char_lit_num = "'\\" ['0'-'9']+ '''
let char_lit_code = "'\\" ['n' 'r' '0'] '''
let char_lit = ''' _ '''

rule token = parse
    blank + 
    { token lexbuf }
  | newline 
      { Location.lineno := (!Location.lineno + 1); token lexbuf }
  | decimal_literal
      { INT_LITERAL( int_of_string(Lexing.lexeme lexbuf) ) }
  | "\""
      {
	let buf = Buffer.create 256 in
	  string_lit buf lexbuf;
	  STR_LITERAL(Buffer.contents buf)
      }
  | char_lit
      {
	let first_char = Lexing.lexeme_char lexbuf 1
	in
	  INT_LITERAL (int_of_char first_char)
      }

  | char_lit_code
      { 
	let code_char = Lexing.lexeme_char lexbuf 2
	in
	  INT_LITERAL (match code_char with
			   'n' -> 0x0a
			 | 'r' -> 0x0d
			 | '0' -> 0x00
			 | _ -> raise (InvalidChar code_char))
      }

  | id_word
      {   
	let id = Lexing.lexeme lexbuf in
	  if Hashtbl.mem keyword_table id then
	    Hashtbl.find keyword_table id
	  else
	    IDENT id
      }

  | '+' { PLUS }
  | '-' { MINUS }
  | '*' { STAR }
  | '/' { SLASH }
  | ';' { SEMI }
  | '(' { LPAR }
  | ')' { RPAR }
  | '[' { LBRACK }
  | ']' { RBRACK }
  | '{' { LBRACE }
  | '}' { RBRACE }
  | ',' { COMMA }
  | '=' { ASIGN }
  | '^' { HUT }
  | '\'' { QUOTE }
  | "&&" { ANDAND }
  | '&' { AND }
  | '|' { OR }
  | '<' { LT }
  | '>' { GT }
  | "||" { OROR }
  | "==" { EQU }
  | "!=" { NEQU }
  | "..." { DOTDOTDOT }

  | eof { EOF }
  | _ { raise( Error("unknown sequens " ^ Lexing.lexeme lexbuf) ) }
  
and string_lit buffer = parse
    '"' { () }
  | _ { Buffer.add_char buffer (nowchar lexbuf); string_lit buffer lexbuf }
