open Unix;;

let parsed_programs = Hashtbl.create 43;;

let compile_file always_compile fname =
  if Hashtbl.mem parsed_programs fname then
    ()
  else
    let out_file = 
      try 
	let last_dot = String.rindex fname '.' in
	let sub_str = (String.sub fname 0 last_dot) in
	  (sub_str^".s",sub_str^".o")
      with Not_found ->
	(fname^".s",fname^".o")
    and lexbuf = Lexing.from_channel (open_in fname) 
    and source_time = ((stat fname).st_mtime) in
      
    let result = Parser.program Lexer.token lexbuf
    in
      Hashtbl.add parsed_programs fname (always_compile,source_time,out_file,result);;

let analyze_program () =
  Hashtbl.iter (fun fname (ac,st,out,program) -> List.iter CodeGen.analyze_type program)
    parsed_programs;
  Hashtbl.iter (fun fname (ac,st,out,program) -> List.iter CodeGen.analyze_global_decl program)
    parsed_programs;;

let do_generate fname (always_compile,source_time,outfile,program) =
  let out_time = 
    if Sys.file_exists (snd outfile) then
      (stat (snd outfile)).st_mtime
    else
      0.0
  in
    if always_compile || (source_time > out_time) then
      let out = open_out( (fst outfile ) )
      in
	ignore (CodeGen.genCode out program);
	close_out out;
	ignore (Unix.system ("gcc -c -o "^(snd outfile)^" "^(fst outfile)))
    else
      ();;
	
let _ =
  Env.push (new Env.symbol_table Env.GlobalTable );
  Env.set_compile_func (compile_file false);
  Array.iter (compile_file true) (Array.sub Sys.argv 1 ((Array.length Sys.argv)-1));
  analyze_program();
  Hashtbl.iter do_generate parsed_programs;;
