open Mcic;;
open Env;;

exception NotType;;
exception ApplyToNotFunction of string;;
exception RefToNotPointer;;

let rec get_typ is_constant get_constant e = 
  match e with
      ID str ->
	(match str with 
	     "int" -> Env.IntType
	   | "char" -> Env.CharType
	   | "void" -> Env.VoidType
	   | "..." -> Env.VarArgType
	   | _ -> prerr_endline"ؤʷ̾"; raise NotType)
    | Expr_Star expr ->
	Pointer_type (get_typ is_constant get_constant expr) 
    | Array_Decl ( expr, elem ) ->
	begin
	  if is_constant elem then
	    Array_type (get_typ is_constant get_constant expr, get_constant elem)
	  else
	    begin
	      prerr_endline("get_typ:礭ʤ");
	      raise NotType
	    end
	end	      
    | _ -> prerr_endline "get_typ type_expr  ޤ";  raise NotType;;

let print_type t =
  match t with 
      CharType -> print_endline "char"
    | IntType -> print_endline "int";
    | Pointer_type p -> print_endline "point";
    | _ -> print_endline "other";;

let type_expr_to_type is_constant get_constant typ =
  get_typ is_constant get_constant typ;;

let rec get_typ_size t =
    match t with
	IntType -> 4
      | CharType -> 1
      | Pointer_type p -> 4
      | Array_type( ref_type, elemcnt ) -> (get_typ_size ref_type) * elemcnt
      | _ -> prerr_endline "get_typ_size"; raise NotType;;

let builtin_func_tbl = 
  Util.create_hashtbl 
    133 
    [( "+" ,Env.IntType );
     ( "-" ,Env.IntType );
     ( "/" ,Env.IntType );
     ( "*" ,Env.IntType )];;

let get_func_rettype func =
  match func with
      ID id -> 
	(if Hashtbl.mem builtin_func_tbl id then
	  Hashtbl.find builtin_func_tbl id
	 else
	   let tbl = Env.find_id id in
	   let var = tbl#get id in
	     match var#typ with
		 Env.FuncType( ret, arg ) -> 
		   ret
	       | _ -> 
		   raise (ApplyToNotFunction(id)))
    | _ -> raise (ApplyToNotFunction("nanika"));;

let get_var_type id =
  let tbl = Env.find_id id in
    (tbl#get id)#typ;;

let rec get_ref_type pointer = 
  match calc_expr_type pointer with
      Pointer_type ref_type -> ref_type
    | Array_type (a_type,nelem) -> a_type
    | _ -> prerr_endline "ݥ󥿤ʤĤ򻲾"; raise RefToNotPointer
	
and calc_expr_type expr =
  match expr with 
      Funcall( func, arg ) -> get_func_rettype func
    | Int_Constant i  -> Env.IntType
    | Str_Constant s -> Env.Pointer_type CharType
    | ID(id) -> get_var_type id
    | Indirect_Ref p -> get_ref_type p
    | Array_Ref (arr,off) -> get_ref_type arr
    | _ -> prerr_endline "get_type"; raise NotType;;

let rec equal_type t1 t2 =
  match (t1,t2) with
      (FuncType (ret1,args1),FuncType (ret2, args2)) ->
	(equal_type ret1 ret2) && 
	(List.for_all2 equal_type args1 args2)
    | (IntType,IntType)
    | (CharType,CharType)
    | (VoidType,VoidType) -> true

    | (Pointer_type p1, Pointer_type p2 ) ->
	equal_type p1 p2 

    | _ -> false;;

let rec aprox_type t1 t2 =		(* ȥݥ󥿤ƱΤȤ *)
  match (t1,t2) with
      ( Pointer_type p, Array_type(arr,off) ) ->
	aprox_type p arr
    | _ -> equal_type t1 t2
